unit uTCPHist;
(*
  ==========================
  TCP Connection history
  ==========================
  Requires : NT4/SP4 or higher, WIN98/WIN98se
  Developed on: D4.03
  Tested on   :  WIN-NT4/SP6, WIN98se, WIN95/OSR1

  ================================================================
                    This software is FREEWARE
                    -------------------------
  If this software works, it was surely written by Dirk Claessens
                   <dirk.claessens16@yucom.be>
               <dirk.claessens.dc@belgium.agfa.com>
  (If it doesn't, I don't know anything about it.)
  ================================================================

  NOTE CAREFULLY:
  ---------------
  TCPHist records the TCP-connection **history** on the system
  it runs on. So it is a *log* rather than a list of  *actual* connections.
  If something was logged, it means there *was* a connection at the given
  Date & time, it does not mean that the connection still exists at the time
  the log is inspected.
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls, ComCtrls, ExtCtrls, IPHelper, Buttons, jpeg;

const
    Version = ' 1.0';
    MaxEntries = 1000;
type
  TForm1 = class(TForm)
    Timer: TTimer;
    sgConnHist: TStringGrid;
    stStart: TStaticText;
    StaticText1: TStaticText;
    sbClear: TSpeedButton;
    cbShowListeners: TCheckBox;
    Image1: TImage;
    procedure FormCreate(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure sbClearClick(Sender: TObject);
    procedure cbShowListenersClick(Sender: TObject);
  private
    { Private declarations }
    function IsNewConnection( Conn: TTcpConnStatus ): boolean;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

//------------------------------------------------------------------------------
procedure TForm1.FormCreate(Sender: TObject);
begin
  Caption := Caption + Version;
  sbClearClick(Sender);
end;

//------------------------------------------------------------------------------
function TForm1.IsNewConnection( Conn: TTcpConnStatus ): boolean;
var
 i : integer;
begin

  // ignore listening ports if option not set
  if not (cbShowListeners.State = cbCHECKED)
  and (Conn.Status = TCPConnState[2]) then
  begin
    Result := false;
    EXIT;
  end;
  //
  Result := true;
  for i := 0 to pred(sgConnHist.RowCount) do
  with sgConnHist do
    if  ( Cells[3,i] = Conn.LocalPort)
    and (Cells[4,i] = Conn.RemoteIP)
    and ( Cells[5,i] = Conn.RemotePort)
    and (Cells[6,i]  = Conn.Status ) then
    begin
      Result := false;
      BREAK;
    end;
end;


//------------------------------------------------------------------------------
procedure TForm1.TimerTimer(Sender: TObject);
var
 List : TList;
 i    : integer;
begin
  Timer.Enabled := false;
  List := TList.Create;
  Get_OpenConnections( List );
  if List.Count > 0 then
   for i := 0 to pred(List.Count) do
   begin
      // IPHelper always reports *all* current connections,
      // so entries already present must be filtered out here
      if IsNewConnection( TTcpConnStatus(List[i]^) ) then
      with TTcpConnStatus( List[i]^), sgConnHist do
      begin
        Cells[0,RowCount-1] := DateToStr(Now);
        Cells[1,RowCount-1] := TimeToStr(Now);
        Cells[2,RowCount-1] := LocalIP;
        Cells[3,RowCount-1] := LocalPort;
        Cells[4,RowCount-1] := RemoteIP;
        Cells[5,RowCount-1] := RemotePort;
        Cells[6,RowCount-1] := Status;
        RowCount := RowCount + 1;
      end;
      Dispose(List[i]); // we _must_ dispose list entry
   end;
  List.Free;
  if sgConnHist.RowCount > MaxEntries then
    if MessageDlg(' There are more then ' + IntToStr(MaxEntries)
                + ' entries in the table!' + #13
                + ' To avoid resource problems, table will be cleared', mtWARNING, [mbOK], 0) = mrOK then
                              sbClearClick(Sender);
  Timer.Enabled := true;
end;

//------------------------------------------------------------------------------
procedure TForm1.sbClearClick(Sender: TObject);
var
 i : integer;
begin
  Timer.Enabled := false;
  with sgConnHist do
  begin
    for i := 0 to pred(RowCount) do
      Rows[i].Clear;
    RowCount := 1;
  end;
  Timer.Enabled := true;
  stStart.Caption := 'TCP connection history since : ' + DateTimeToStr(now);
  cbShowListenersClick(Sender);
end;

//------------------------------------------------------------------------------
procedure TForm1.cbShowListenersClick(Sender: TObject);
begin
  with sgConnHist do
  begin
   Cells[0,RowCount-1] := DateToStr(Now);
   Cells[1,RowCount-1] := TimeToStr(Now);
   Cells[2,RowCount-1] := 'LISTENING PORTS';
   case cbShowListeners.State of
    cbCHECKED   : Cells[3,RowCount-1] := '  ON';
    cbUNCHECKED : Cells[3,RowCount-1] := '  OFF';
   end;
   RowCount := RowCount + 1;
  end;
end;

end.
